package com.bipt.intelligentapplicationorchestrationservice.entity.vo;

import com.bipt.intelligentapplicationorchestrationservice.entity.enums.ErrorCodeEnum;


import java.io.Serializable;

public class ResponseVO<T> implements Serializable {
    private Integer code;   //状态码
    private String message; //描述信息
    private T data;         //业务数据


    //私有构造方法
    private ResponseVO(Integer code, String message, T data) {
        this.code = code;
        this.message = message;
        this.data = data;
    }

    //静态工厂方法
    //成功响应（无数据）
    public static <T> ResponseVO<T> success() {
        return new ResponseVO<>(200, "OK", null);
    }
    //成功响应（有数据）
    public static <T> ResponseVO<T> success(T data) {
        return new ResponseVO<>(200, "OK", data);
    }
    //失败响应(自定义错误码和消息)
    public static <T> ResponseVO<T> error(Integer code, String message) {
        return new ResponseVO<>(code, message, null);
    }
    //失败响应（基于预定义错误枚举）
    public static <T> ResponseVO<T> error(ErrorCodeEnum errorCode) {
       return new ResponseVO<>(errorCode.getCode(), errorCode.getMessage(), null);
    }
}
