package com.bipt.intelligentapplicationorchestrationservice.mapper;

import com.bipt.intelligentapplicationorchestrationservice.entity.entity.GpuResource;
import org.apache.ibatis.annotations.*;
import org.apache.ibatis.jdbc.SQL;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

@Mapper
public interface GpuResourceDao {
    //---------------------- 基础CRUD ------------------------
    @Insert("INSERT INTO Ipz.public.gpu_resource (GPUModel, GPUMemorySize, Ip) " +
            "VALUES (#{model}, #{memory}, #{ip})")
    @Options(useGeneratedKeys = true, keyProperty = "GPUId")
    Integer insert(GpuResource entity);

    //物理删除
    @Delete("DELETE FROM Ipz.public.gpu_resource WHERE GPUId = #{gpuId}")
    Integer deleteById(@Param("gpuId") Long gpuId);

    // 逻辑删除
    @Update("UPDATE Ipz.public.gpu_resource" +
            " SET is_deleted = 1, update_time = NOW() " +
            " WHERE GPUId = #{gpuId}")
    Integer isDeleted(@Param("gpuId") Long gpuId);

    @Update("UPDATE Ipz.public.gpu_resource " +
            "SET GPUModel = #{model}, GPUMemorySize = #{memory}, Ip = #{ip} " +
            "WHERE GPUId = #{GPUId}")
    Integer updateById(GpuResource entity);

    @Select("SELECT * FROM Ipz.public.gpu_resource WHERE GPUId = #{gpuId} AND is_deleted = 0")
    GpuResource selectById(@Param("gpuId") Long gpuId);

    //---------------------- 缓存相关扩展 ------------------------

    /**
     * 分页全量查询（缓存初始化用）
     * @param offset 起始位置
     * @param limit 每页数量
     */
//    @Select("SELECT * FROM ipz.gpu_resource " +
//            "ORDER BY GPUId ASC LIMIT #{limit} OFFSET #{offset}")
    List<GpuResource> findByPage(@Param("offset") int offset,
                                 @Param("limit") int limit);

    /**
     * 增量数据查询（缓存同步用）
     * @param since 起始时间
     */
//    @Select("SELECT *, is_deleted FROM ipz.gpu_resource " +
//            "WHERE update_time > #{since} " +
//            "ORDER BY update_time ASC")
    List<GpuResource> findModifiedSince(@Param("since") LocalDateTime since);

    /**
     * 带锁查询（防缓存击穿）
     */
//    @Select("SELECT * FROM ipz.gpu_resource " +
//            "WHERE GPUId = #{gpuId} FOR UPDATE NOWAIT")
    GpuResource selectByIdWithLock(@Param("gpuId") Long gpuId);

    /**
     * 动态条件查询（管理界面筛选用）
     */
    // @SelectProvider(type = GpuSqlBuilder.class, method = "buildDynamicQuery")
    List<GpuResource> selectByFields(@Param("params") Map<String, Object> params);
}

// 动态SQL构造器
class GpuSqlBuilder {
    public static String buildDynamicQuery(Map<String, Object> params) {
        return new SQL() {{
            SELECT("*");
            FROM("Ipz.public.gpu_resource");
            if (params.containsKey("model")) {
                WHERE("GPUModel LIKE #{params.model}");
            }
            if (params.containsKey("memoryMin")) {
                WHERE("GPUMemorySize >= #{params.memoryMin}");
            }
            if (params.containsKey("ip")) {
                WHERE("Ip = #{params.ip}");
            }
            if (params.containsKey("isDeleted")) {
                WHERE("is_deleted = #{params.isDeleted}");
            }
        }}.toString();
    }

}
