package com.bipt.intelligentapplicationorchestrationservice.service.Impl;

import com.bipt.intelligentapplicationorchestrationservice.mapper.GpuResourceDao;
import com.bipt.intelligentapplicationorchestrationservice.mapper.GpuMapper;
import com.bipt.intelligentapplicationorchestrationservice.entity.dto.GpuCreateDTO;
import com.bipt.intelligentapplicationorchestrationservice.entity.dto.GpuResponseDTO;
import com.bipt.intelligentapplicationorchestrationservice.entity.dto.GpuUpdateDTO;
import com.bipt.intelligentapplicationorchestrationservice.entity.entity.GpuResource;
import com.bipt.intelligentapplicationorchestrationservice.entity.enums.ErrorCodeEnum;
import com.bipt.intelligentapplicationorchestrationservice.entity.vo.ResponseVO;
import com.bipt.intelligentapplicationorchestrationservice.service.GpuManageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class GpuManageServiceImpl implements GpuManageService {
    @Autowired
    private GpuResourceDao gpuDao;

    @Autowired
    private GpuMapper gpuMapper;
    @Autowired
    private GpuResourceDao gpuResourceDao;

    @Transactional
    //创建GPU资源
    public ResponseVO createGpuResource(GpuCreateDTO dto) {
        GpuResource entity = gpuMapper.toEntity(dto);
        gpuDao.insert(entity);
        return ResponseVO.success(entity);
    }

    @Transactional
    //删除GPU资源（逻辑删除）
    public ResponseVO deleteGpuResource(Long gpuId) {
        GpuResource entity = gpuDao.selectById(gpuId);
        if (entity == null) {
            return ResponseVO.error(ErrorCodeEnum.GPU_NOT_FOUND);
        }
        gpuDao.isDeleted(gpuId);
        return ResponseVO.success();
    }

    @Transactional
    //更新GPU资源
    public void updateGpuResource(GpuUpdateDTO dto) {
        GpuResource entity = gpuMapper.toEntity(dto);
        gpuDao.updateById(entity);
    }

    @Override
    //模糊匹配查询
    public List<GpuResponseDTO> searchByCriteria(String model, Integer memorySize, String ip) {
        // PermissionCheckUtil.checkTenantAccess();

        Map<String, Object> params = new HashMap<>();
        if(model != null) params.put("model","%" + model + "%");
        if(memorySize!=null) params.put("memorySize", memorySize);
        if(ip!=null) params.put("ip", ip);

        List<GpuResource> entities = gpuResourceDao.selectByFields(params);

        return entities.stream().map(gpuMapper::toDTO).collect(Collectors.toList());

    }
}
